@@ -3,6 +3,8 @@
 
 """Tests for coverage.data"""
 
+from __future__ import annotations
+
 import glob
 import os
 import os.path
@@ -10,7 +12,11 @@ import re
 import sqlite3
 import threading
 
+from typing import (
+    Any, Callable, Collection, Dict, Iterable, Mapping, Set, TypeVar, Union
+)
 from unittest import mock
+
 import pytest
 
 from coverage.data import CoverageData, combine_parallel_data
@@ -18,6 +24,7 @@ from coverage.data import add_data_to_hash, line_counts
 from coverage.debug import DebugControlString
 from coverage.exceptions import DataError, NoDataError
 from coverage.files import PathAliases, canonical_filename
+from coverage.types import TArc, TLineNo
 
 from tests.coveragetest import CoverageTest
 from tests.helpers import assert_count_equal
@@ -58,7 +65,7 @@ SUMMARY_3_4 = {'x.py': 4, 'y.py': 2, 'z.py': 1}
 MEASURED_FILES_3_4 = ['x.py', 'y.py', 'z.py']
 
 
-def DebugCoverageData(*args, **kwargs):
+def DebugCoverageData(*args: Any, **kwargs: Any) -> CoverageData:
     """Factory for CovergeData instances with debugging turned on.
 
     This lets us exercise the debugging lines in sqldata.py.  We don't make
@@ -73,25 +80,31 @@ def DebugCoverageData(*args, **kwargs):
         # This is a way to get a mix of debug options across the tests.
         options.extend(["sqldata"])
     debug = DebugControlString(options=options)
-    return CoverageData(*args, debug=debug, **kwargs)
+    return CoverageData(*args, debug=debug, **kwargs)   # type: ignore[misc]
 
 
-def assert_line_counts(covdata, counts, fullpath=False):
+TCoverageData = Callable[..., CoverageData]
+
+def assert_line_counts(
+    covdata: CoverageData,
+    counts: Mapping[str, int],
+    fullpath: bool = False,
+) -> None:
     """Check that the line_counts of `covdata` is `counts`."""
     assert line_counts(covdata, fullpath) == counts
 
-def assert_measured_files(covdata, measured):
+def assert_measured_files(covdata: CoverageData, measured: Iterable[str]) -> None:
     """Check that `covdata`'s measured files are `measured`."""
     assert_count_equal(covdata.measured_files(), measured)
 
-def assert_lines1_data(covdata):
+def assert_lines1_data(covdata: CoverageData) -> None:
     """Check that `covdata` has the data from LINES1."""
     assert_line_counts(covdata, SUMMARY_1)
     assert_measured_files(covdata, MEASURED_FILES_1)
     assert_count_equal(covdata.lines("a.py"), A_PY_LINES_1)
     assert not covdata.has_arcs()
 
-def assert_arcs3_data(covdata):
+def assert_arcs3_data(covdata: CoverageData) -> None:
     """Check that `covdata` has the data from ARCS3."""
     assert_line_counts(covdata, SUMMARY_3)
     assert_measured_files(covdata, MEASURED_FILES_3)
@@ -102,7 +115,9 @@ def assert_arcs3_data(covdata):
     assert covdata.has_arcs()
 
 
-def dicts_from_sets(file_data):
+TData = TypeVar("TData", bound=Union[TLineNo, TArc])
+
+def dicts_from_sets(file_data: Dict[str, Set[TData]]) -> Dict[str, Dict[TData, None]]:
     """Convert a dict of sets into a dict of dicts.
 
     Before 6.0, file data was a dict with None as the values.  In 6.0, file
@@ -115,57 +130,57 @@ def dicts_from_sets(file_data):
 class CoverageDataTest(CoverageTest):
     """Test cases for CoverageData."""
 
-    def test_empty_data_is_false(self):
+    def test_empty_data_is_false(self) -> None:
         covdata = DebugCoverageData()
         assert not covdata
         self.assert_doesnt_exist(".coverage")
 
-    def test_empty_data_is_false_when_read(self):
+    def test_empty_data_is_false_when_read(self) -> None:
         covdata = DebugCoverageData()
         covdata.read()
         assert not covdata
         self.assert_doesnt_exist(".coverage")
 
-    def test_line_data_is_true(self):
+    def test_line_data_is_true(self) -> None:
         covdata = DebugCoverageData()
         covdata.add_lines(LINES_1)
         assert covdata
 
-    def test_arc_data_is_true(self):
+    def test_arc_data_is_true(self) -> None:
         covdata = DebugCoverageData()
         covdata.add_arcs(ARCS_3)
         assert covdata
 
-    def test_empty_line_data_is_false(self):
+    def test_empty_line_data_is_false(self) -> None:
         covdata = DebugCoverageData()
         covdata.add_lines({})
         assert not covdata
 
-    def test_empty_arc_data_is_false(self):
+    def test_empty_arc_data_is_false(self) -> None:
         covdata = DebugCoverageData()
         covdata.add_arcs({})
         assert not covdata
 
     @pytest.mark.parametrize("lines", [LINES_1, dicts_from_sets(LINES_1)])
-    def test_adding_lines(self, lines):
+    def test_adding_lines(self, lines: Mapping[str, Collection[TLineNo]]) -> None:
         covdata = DebugCoverageData()
         covdata.add_lines(lines)
         assert_lines1_data(covdata)
 
     @pytest.mark.parametrize("arcs", [ARCS_3, dicts_from_sets(ARCS_3)])
-    def test_adding_arcs(self, arcs):
+    def test_adding_arcs(self, arcs: Mapping[str, Collection[TArc]]) -> None:
         covdata = DebugCoverageData()
         covdata.add_arcs(arcs)
         assert_arcs3_data(covdata)
 
-    def test_ok_to_add_lines_twice(self):
+    def test_ok_to_add_lines_twice(self) -> None:
         covdata = DebugCoverageData()
         covdata.add_lines(LINES_1)
         covdata.add_lines(LINES_2)
         assert_line_counts(covdata, SUMMARY_1_2)
         assert_measured_files(covdata, MEASURED_FILES_1_2)
 
-    def test_ok_to_add_arcs_twice(self):
+    def test_ok_to_add_arcs_twice(self) -> None:
         covdata = DebugCoverageData()
         covdata.add_arcs(ARCS_3)
         covdata.add_arcs(ARCS_4)
@@ -173,7 +188,7 @@ class CoverageDataTest(CoverageTest):
         assert_measured_files(covdata, MEASURED_FILES_3_4)
 
     @pytest.mark.parametrize("klass", [CoverageData, DebugCoverageData])
-    def test_cant_add_arcs_with_lines(self, klass):
+    def test_cant_add_arcs_with_lines(self, klass: TCoverageData) -> None:
         covdata = klass()
         covdata.add_lines(LINES_1)
         msg = "Can't add branch measurements to existing line data"
@@ -181,26 +196,26 @@ class CoverageDataTest(CoverageTest):
             covdata.add_arcs(ARCS_3)
 
     @pytest.mark.parametrize("klass", [CoverageData, DebugCoverageData])
-    def test_cant_add_lines_with_arcs(self, klass):
+    def test_cant_add_lines_with_arcs(self, klass: TCoverageData) -> None:
         covdata = klass()
         covdata.add_arcs(ARCS_3)
         msg = "Can't add line measurements to existing branch data"
         with pytest.raises(DataError, match=msg):
             covdata.add_lines(LINES_1)
 
-    def test_touch_file_with_lines(self):
+    def test_touch_file_with_lines(self) -> None:
         covdata = DebugCoverageData()
         covdata.add_lines(LINES_1)
         covdata.touch_file('zzz.py')
         assert_measured_files(covdata, MEASURED_FILES_1 + ['zzz.py'])
 
-    def test_touch_file_with_arcs(self):
+    def test_touch_file_with_arcs(self) -> None:
         covdata = DebugCoverageData()
         covdata.add_arcs(ARCS_3)
         covdata.touch_file('zzz.py')
         assert_measured_files(covdata, MEASURED_FILES_3 + ['zzz.py'])
 
-    def test_set_query_contexts(self):
+    def test_set_query_contexts(self) -> None:
         covdata = DebugCoverageData()
         covdata.set_context('test_a')
         covdata.add_lines(LINES_1)
@@ -209,14 +224,14 @@ class CoverageDataTest(CoverageTest):
         covdata.set_query_contexts(['other'])
         assert covdata.lines('a.py') == []
 
-    def test_no_lines_vs_unmeasured_file(self):
+    def test_no_lines_vs_unmeasured_file(self) -> None:
         covdata = DebugCoverageData()
         covdata.add_lines(LINES_1)
         covdata.touch_file('zzz.py')
         assert covdata.lines('zzz.py') == []
         assert covdata.lines('no_such_file.py') is None
 
-    def test_lines_with_contexts(self):
+    def test_lines_with_contexts(self) -> None:
         covdata = DebugCoverageData()
         covdata.set_context('test_a')
         covdata.add_lines(LINES_1)
@@ -226,7 +241,7 @@ class CoverageDataTest(CoverageTest):
         covdata.set_query_contexts(['other'])
         assert covdata.lines('a.py') == []
 
-    def test_contexts_by_lineno_with_lines(self):
+    def test_contexts_by_lineno_with_lines(self) -> None:
         covdata = DebugCoverageData()
         covdata.set_context('test_a')
         covdata.add_lines(LINES_1)
@@ -234,7 +249,7 @@ class CoverageDataTest(CoverageTest):
         assert covdata.contexts_by_lineno('a.py') == expected
 
     @pytest.mark.parametrize("lines", [LINES_1, dicts_from_sets(LINES_1)])
-    def test_no_duplicate_lines(self, lines):
+    def test_no_duplicate_lines(self, lines: Mapping[str, Collection[TLineNo]]) -> None:
         covdata = DebugCoverageData()
         covdata.set_context("context1")
         covdata.add_lines(lines)
@@ -243,7 +258,7 @@ class CoverageDataTest(CoverageTest):
         assert covdata.lines('a.py') == A_PY_LINES_1
 
     @pytest.mark.parametrize("arcs", [ARCS_3, dicts_from_sets(ARCS_3)])
-    def test_no_duplicate_arcs(self, arcs):
+    def test_no_duplicate_arcs(self, arcs: Mapping[str, Collection[TArc]]) -> None:
         covdata = DebugCoverageData()
         covdata.set_context("context1")
         covdata.add_arcs(arcs)
@@ -251,7 +266,7 @@ class CoverageDataTest(CoverageTest):
         covdata.add_arcs(arcs)
         assert covdata.arcs('x.py') == X_PY_ARCS_3
 
-    def test_no_arcs_vs_unmeasured_file(self):
+    def test_no_arcs_vs_unmeasured_file(self) -> None:
         covdata = DebugCoverageData()
         covdata.add_arcs(ARCS_3)
         covdata.touch_file('zzz.py')
@@ -260,7 +275,7 @@ class CoverageDataTest(CoverageTest):
         assert covdata.arcs('zzz.py') == []
         assert covdata.arcs('no_such_file.py') is None
 
-    def test_arcs_with_contexts(self):
+    def test_arcs_with_contexts(self) -> None:
         covdata = DebugCoverageData()
         covdata.set_context('test_x')
         covdata.add_arcs(ARCS_3)
@@ -270,20 +285,20 @@ class CoverageDataTest(CoverageTest):
         covdata.set_query_contexts(['other'])
         assert covdata.arcs('x.py') == []
 
-    def test_contexts_by_lineno_with_arcs(self):
+    def test_contexts_by_lineno_with_arcs(self) -> None:
         covdata = DebugCoverageData()
         covdata.set_context('test_x')
         covdata.add_arcs(ARCS_3)
         expected = {1: ['test_x'], 2: ['test_x'], 3: ['test_x']}
         assert covdata.contexts_by_lineno('x.py') == expected
 
-    def test_contexts_by_lineno_with_unknown_file(self):
+    def test_contexts_by_lineno_with_unknown_file(self) -> None:
         covdata = DebugCoverageData()
         covdata.set_context('test_x')
         covdata.add_arcs(ARCS_3)
         assert covdata.contexts_by_lineno('xyz.py') == {}
 
-    def test_context_by_lineno_with_query_contexts_with_lines(self):
+    def test_context_by_lineno_with_query_contexts_with_lines(self) -> None:
         covdata = DebugCoverageData()
         covdata.set_context("test_1")
         covdata.add_lines(LINES_1)
@@ -292,7 +307,7 @@ class CoverageDataTest(CoverageTest):
         covdata.set_query_context("test_1")
         assert covdata.contexts_by_lineno("a.py") == dict.fromkeys([1,2], ["test_1"])
 
-    def test_context_by_lineno_with_query_contexts_with_arcs(self):
+    def test_context_by_lineno_with_query_contexts_with_arcs(self) -> None:
         covdata = DebugCoverageData()
         covdata.set_context("test_1")
         covdata.add_arcs(ARCS_3)
@@ -301,7 +316,7 @@ class CoverageDataTest(CoverageTest):
         covdata.set_query_context("test_1")
         assert covdata.contexts_by_lineno("x.py") == dict.fromkeys([1,2,3], ["test_1"])
 
-    def test_file_tracer_name(self):
+    def test_file_tracer_name(self) -> None:
         covdata = DebugCoverageData()
         covdata.add_lines({
             "p1.foo": [1, 2, 3],
@@ -314,7 +329,7 @@ class CoverageDataTest(CoverageTest):
         assert covdata.file_tracer("main.py") == ""
         assert covdata.file_tracer("p3.not_here") is None
 
-    def test_ok_to_repeat_file_tracer(self):
+    def test_ok_to_repeat_file_tracer(self) -> None:
         covdata = DebugCoverageData()
         covdata.add_lines({
             "p1.foo": [1, 2, 3],
@@ -324,7 +339,7 @@ class CoverageDataTest(CoverageTest):
         covdata.add_file_tracers({"p1.foo": "p1.plugin"})
         assert covdata.file_tracer("p1.foo") == "p1.plugin"
 
-    def test_ok_to_set_empty_file_tracer(self):
+    def test_ok_to_set_empty_file_tracer(self) -> None:
         covdata = DebugCoverageData()
         covdata.add_lines({
             "p1.foo": [1, 2, 3],
@@ -335,7 +350,7 @@ class CoverageDataTest(CoverageTest):
         assert covdata.file_tracer("p1.foo") == "p1.plugin"
         assert covdata.file_tracer("main.py") == ""
 
-    def test_cant_file_tracer_unmeasured_files(self):
+    def test_cant_file_tracer_unmeasured_files(self) -> None:
         covdata = DebugCoverageData()
         msg = "Can't add file tracer data for unmeasured file 'p1.foo'"
         with pytest.raises(DataError, match=msg):
@@ -345,7 +360,7 @@ class CoverageDataTest(CoverageTest):
         with pytest.raises(DataError, match=msg):
             covdata.add_file_tracers({"p1.foo": "p1.plugin"})
 
-    def test_cant_change_file_tracer_name(self):
+    def test_cant_change_file_tracer_name(self) -> None:
         covdata = DebugCoverageData()
         covdata.add_lines({"p1.foo": [1, 2, 3]})
         covdata.add_file_tracers({"p1.foo": "p1.plugin"})
@@ -354,7 +369,7 @@ class CoverageDataTest(CoverageTest):
         with pytest.raises(DataError, match=msg):
             covdata.add_file_tracers({"p1.foo": "p1.plugin.foo"})
 
-    def test_update_lines(self):
+    def test_update_lines(self) -> None:
         covdata1 = DebugCoverageData(suffix='1')
         covdata1.add_lines(LINES_1)
 
@@ -368,7 +383,7 @@ class CoverageDataTest(CoverageTest):
         assert_line_counts(covdata3, SUMMARY_1_2)
         assert_measured_files(covdata3, MEASURED_FILES_1_2)
 
-    def test_update_arcs(self):
+    def test_update_arcs(self) -> None:
         covdata1 = DebugCoverageData(suffix='1')
         covdata1.add_arcs(ARCS_3)
 
@@ -382,7 +397,7 @@ class CoverageDataTest(CoverageTest):
         assert_line_counts(covdata3, SUMMARY_3_4)
         assert_measured_files(covdata3, MEASURED_FILES_3_4)
 
-    def test_update_cant_mix_lines_and_arcs(self):
+    def test_update_cant_mix_lines_and_arcs(self) -> None:
         covdata1 = DebugCoverageData(suffix='1')
         covdata1.add_lines(LINES_1)
 
@@ -395,7 +410,7 @@ class CoverageDataTest(CoverageTest):
         with pytest.raises(DataError, match="Can't combine line data with arc data"):
             covdata2.update(covdata1)
 
-    def test_update_file_tracers(self):
+    def test_update_file_tracers(self) -> None:
         covdata1 = DebugCoverageData(suffix='1')
         covdata1.add_lines({
             "p1.html": [1, 2, 3, 4],
@@ -428,7 +443,7 @@ class CoverageDataTest(CoverageTest):
         assert covdata3.file_tracer("p3.foo") == "foo_plugin"
         assert covdata3.file_tracer("main.py") == ""
 
-    def test_update_conflicting_file_tracers(self):
+    def test_update_conflicting_file_tracers(self) -> None:
         covdata1 = DebugCoverageData(suffix='1')
         covdata1.add_lines({"p1.html": [1, 2, 3]})
         covdata1.add_file_tracers({"p1.html": "html.plugin"})
@@ -445,7 +460,7 @@ class CoverageDataTest(CoverageTest):
         with pytest.raises(DataError, match=msg):
             covdata2.update(covdata1)
 
-    def test_update_file_tracer_vs_no_file_tracer(self):
+    def test_update_file_tracer_vs_no_file_tracer(self) -> None:
         covdata1 = DebugCoverageData(suffix="1")
         covdata1.add_lines({"p1.html": [1, 2, 3]})
         covdata1.add_file_tracers({"p1.html": "html.plugin"})
@@ -461,7 +476,7 @@ class CoverageDataTest(CoverageTest):
         with pytest.raises(DataError, match=msg):
             covdata2.update(covdata1)
 
-    def test_update_lines_empty(self):
+    def test_update_lines_empty(self) -> None:
         covdata1 = DebugCoverageData(suffix='1')
         covdata1.add_lines(LINES_1)
 
@@ -469,7 +484,7 @@ class CoverageDataTest(CoverageTest):
         covdata1.update(covdata2)
         assert_line_counts(covdata1, SUMMARY_1)
 
-    def test_update_arcs_empty(self):
+    def test_update_arcs_empty(self) -> None:
         covdata1 = DebugCoverageData(suffix='1')
         covdata1.add_arcs(ARCS_3)
 
@@ -477,14 +492,14 @@ class CoverageDataTest(CoverageTest):
         covdata1.update(covdata2)
         assert_line_counts(covdata1, SUMMARY_3)
 
-    def test_asking_isnt_measuring(self):
+    def test_asking_isnt_measuring(self) -> None:
         # Asking about an unmeasured file shouldn't make it seem measured.
         covdata = DebugCoverageData()
         assert_measured_files(covdata, [])
         assert covdata.arcs("missing.py") is None
         assert_measured_files(covdata, [])
 
-    def test_add_to_hash_with_lines(self):
+    def test_add_to_hash_with_lines(self) -> None:
         covdata = DebugCoverageData()
         covdata.add_lines(LINES_1)
         hasher = mock.Mock()
@@ -494,7 +509,7 @@ class CoverageDataTest(CoverageTest):
             mock.call.update(""),       # file_tracer name
         ]
 
-    def test_add_to_hash_with_arcs(self):
+    def test_add_to_hash_with_arcs(self) -> None:
         covdata = DebugCoverageData()
         covdata.add_arcs(ARCS_3)
         covdata.add_file_tracers({"y.py": "hologram_plugin"})
@@ -505,7 +520,7 @@ class CoverageDataTest(CoverageTest):
             mock.call.update("hologram_plugin"),                # file_tracer name
         ]
 
-    def test_add_to_lines_hash_with_missing_file(self):
+    def test_add_to_lines_hash_with_missing_file(self) -> None:
         # https://github.com/nedbat/coveragepy/issues/403
         covdata = DebugCoverageData()
         covdata.add_lines(LINES_1)
@@ -516,7 +531,7 @@ class CoverageDataTest(CoverageTest):
             mock.call.update(None),
         ]
 
-    def test_add_to_arcs_hash_with_missing_file(self):
+    def test_add_to_arcs_hash_with_missing_file(self) -> None:
         # https://github.com/nedbat/coveragepy/issues/403
         covdata = DebugCoverageData()
         covdata.add_arcs(ARCS_3)
@@ -528,25 +543,25 @@ class CoverageDataTest(CoverageTest):
             mock.call.update(None),
         ]
 
-    def test_empty_lines_are_still_lines(self):
+    def test_empty_lines_are_still_lines(self) -> None:
         covdata = DebugCoverageData()
         covdata.add_lines({})
         covdata.touch_file("abc.py")
         assert not covdata.has_arcs()
 
-    def test_empty_arcs_are_still_arcs(self):
+    def test_empty_arcs_are_still_arcs(self) -> None:
         covdata = DebugCoverageData()
         covdata.add_arcs({})
         covdata.touch_file("abc.py")
         assert covdata.has_arcs()
 
-    def test_cant_touch_in_empty_data(self):
+    def test_cant_touch_in_empty_data(self) -> None:
         covdata = DebugCoverageData()
         msg = "Can't touch files in an empty CoverageData"
         with pytest.raises(DataError, match=msg):
             covdata.touch_file("abc.py")
 
-    def test_read_and_write_are_opposites(self):
+    def test_read_and_write_are_opposites(self) -> None:
         covdata1 = DebugCoverageData()
         covdata1.add_arcs(ARCS_3)
         covdata1.write()
@@ -555,11 +570,11 @@ class CoverageDataTest(CoverageTest):
         covdata2.read()
         assert_arcs3_data(covdata2)
 
-    def test_thread_stress(self):
+    def test_thread_stress(self) -> None:
         covdata = DebugCoverageData()
         exceptions = []
 
-        def thread_main():
+        def thread_main() -> None:
             """Every thread will try to add the same data."""
             try:
                 covdata.add_lines(LINES_1)
@@ -579,7 +594,7 @@ class CoverageDataTest(CoverageTest):
 class CoverageDataInTempDirTest(CoverageTest):
     """Tests of CoverageData that need a temporary directory to make files."""
 
-    def test_read_write_lines(self):
+    def test_read_write_lines(self) -> None:
         covdata1 = DebugCoverageData("lines.dat")
         covdata1.add_lines(LINES_1)
         covdata1.write()
@@ -588,7 +603,7 @@ class CoverageDataInTempDirTest(CoverageTest):
         covdata2.read()
         assert_lines1_data(covdata2)
 
-    def test_read_write_arcs(self):
+    def test_read_write_arcs(self) -> None:
         covdata1 = DebugCoverageData("arcs.dat")
         covdata1.add_arcs(ARCS_3)
         covdata1.write()
@@ -597,14 +612,14 @@ class CoverageDataInTempDirTest(CoverageTest):
         covdata2.read()
         assert_arcs3_data(covdata2)
 
-    def test_read_errors(self):
+    def test_read_errors(self) -> None:
         self.make_file("xyzzy.dat", "xyzzy")
         with pytest.raises(DataError, match=r"Couldn't .* '.*[/\\]xyzzy.dat': \S+"):
             covdata = DebugCoverageData("xyzzy.dat")
             covdata.read()
         assert not covdata
 
-    def test_hard_read_error(self):
+    def test_hard_read_error(self) -> None:
         self.make_file("noperms.dat", "go away")
         os.chmod("noperms.dat", 0)
         with pytest.raises(DataError, match=r"Couldn't .* '.*[/\\]noperms.dat': \S+"):
@@ -612,17 +627,17 @@ class CoverageDataInTempDirTest(CoverageTest):
             covdata.read()
 
     @pytest.mark.parametrize("klass", [CoverageData, DebugCoverageData])
-    def test_error_when_closing(self, klass):
+    def test_error_when_closing(self, klass: TCoverageData) -> None:
         msg = r"Couldn't .* '.*[/\\]flaked.dat': \S+"
         with pytest.raises(DataError, match=msg):
             covdata = klass("flaked.dat")
             covdata.add_lines(LINES_1)
             # I don't know how to make a real error, so let's fake one.
             sqldb = list(covdata._dbs.values())[0]
-            sqldb.close = lambda: 1/0
+            sqldb.close = lambda: 1/0       # type: ignore[assignment]
             covdata.add_lines(LINES_1)
 
-    def test_wrong_schema_version(self):
+    def test_wrong_schema_version(self) -> None:
         with sqlite3.connect("wrong_schema.db") as con:
             con.execute("create table coverage_schema (version integer)")
             con.execute("insert into coverage_schema (version) values (99)")
@@ -632,7 +647,7 @@ class CoverageDataInTempDirTest(CoverageTest):
             covdata.read()
         assert not covdata
 
-    def test_wrong_schema_schema(self):
+    def test_wrong_schema_schema(self) -> None:
         with sqlite3.connect("wrong_schema_schema.db") as con:
             con.execute("create table coverage_schema (xyzzy integer)")
             con.execute("insert into coverage_schema (xyzzy) values (99)")
@@ -646,13 +661,13 @@ class CoverageDataInTempDirTest(CoverageTest):
 class CoverageDataFilesTest(CoverageTest):
     """Tests of CoverageData file handling."""
 
-    def test_reading_missing(self):
+    def test_reading_missing(self) -> None:
         self.assert_doesnt_exist(".coverage")
         covdata = DebugCoverageData()
         covdata.read()
         assert_line_counts(covdata, {})
 
-    def test_writing_and_reading(self):
+    def test_writing_and_reading(self) -> None:
         covdata1 = DebugCoverageData()
         covdata1.add_lines(LINES_1)
         covdata1.write()
@@ -661,7 +676,7 @@ class CoverageDataFilesTest(CoverageTest):
         covdata2.read()
         assert_line_counts(covdata2, SUMMARY_1)
 
-    def test_debug_output_with_debug_option(self):
+    def test_debug_output_with_debug_option(self) -> None:
         # With debug option dataio, we get debug output about reading and
         # writing files.
         debug = DebugControlString(options=["dataio"])
@@ -681,7 +696,7 @@ class CoverageDataFilesTest(CoverageTest):
             debug.get_output()
         )
 
-    def test_debug_output_without_debug_option(self):
+    def test_debug_output_without_debug_option(self) -> None:
         # With a debug object, but not the dataio option, we don't get debug
         # output.
         debug = DebugControlString(options=[])
@@ -695,7 +710,7 @@ class CoverageDataFilesTest(CoverageTest):
 
         assert debug.get_output() == ""
 
-    def test_explicit_suffix(self):
+    def test_explicit_suffix(self) -> None:
         self.assert_doesnt_exist(".coverage.SUFFIX")
         covdata = DebugCoverageData(suffix='SUFFIX')
         covdata.add_lines(LINES_1)
@@ -703,7 +718,7 @@ class CoverageDataFilesTest(CoverageTest):
         self.assert_exists(".coverage.SUFFIX")
         self.assert_doesnt_exist(".coverage")
 
-    def test_true_suffix(self):
+    def test_true_suffix(self) -> None:
         self.assert_file_count(".coverage.*", 0)
 
         # suffix=True will make a randomly named data file.
@@ -725,7 +740,7 @@ class CoverageDataFilesTest(CoverageTest):
         # In addition to being different, the suffixes have the pid in them.
         assert all(str(os.getpid()) in fn for fn in data_files2)
 
-    def test_combining(self):
+    def test_combining(self) -> None:
         self.assert_file_count(".coverage.*", 0)
 
         covdata1 = DebugCoverageData(suffix='1')
@@ -746,7 +761,7 @@ class CoverageDataFilesTest(CoverageTest):
         assert_measured_files(covdata3, MEASURED_FILES_1_2)
         self.assert_file_count(".coverage.*", 0)
 
-    def test_erasing(self):
+    def test_erasing(self) -> None:
         covdata1 = DebugCoverageData()
         covdata1.add_lines(LINES_1)
         covdata1.write()
@@ -758,7 +773,7 @@ class CoverageDataFilesTest(CoverageTest):
         covdata2.read()
         assert_line_counts(covdata2, {})
 
-    def test_erasing_parallel(self):
+    def test_erasing_parallel(self) -> None:
         self.make_file("datafile.1")
         self.make_file("datafile.2")
         self.make_file(".coverage")
@@ -767,7 +782,7 @@ class CoverageDataFilesTest(CoverageTest):
         self.assert_file_count("datafile.*", 0)
         self.assert_exists(".coverage")
 
-    def test_combining_with_aliases(self):
+    def test_combining_with_aliases(self) -> None:
         covdata1 = DebugCoverageData(suffix='1')
         covdata1.add_lines({
             '/home/ned/proj/src/a.py': {1, 2},
@@ -807,7 +822,7 @@ class CoverageDataFilesTest(CoverageTest):
         assert_measured_files(covdata3, [apy, sub_bpy, template_html])
         assert covdata3.file_tracer(template_html) == 'html.plugin'
 
-    def test_combining_from_different_directories(self):
+    def test_combining_from_different_directories(self) -> None:
         os.makedirs('cov1')
         covdata1 = DebugCoverageData('cov1/.coverage.1')
         covdata1.add_lines(LINES_1)
@@ -832,7 +847,7 @@ class CoverageDataFilesTest(CoverageTest):
         self.assert_doesnt_exist("cov2/.coverage.2")
         self.assert_exists(".coverage.xxx")
 
-    def test_combining_from_files(self):
+    def test_combining_from_files(self) -> None:
         os.makedirs('cov1')
         covdata1 = DebugCoverageData('cov1/.coverage.1')
         covdata1.add_lines(LINES_1)
@@ -862,13 +877,13 @@ class CoverageDataFilesTest(CoverageTest):
         self.assert_exists(".coverage.xxx")
         self.assert_exists("cov2/.coverage.xxx")
 
-    def test_combining_from_nonexistent_directories(self):
+    def test_combining_from_nonexistent_directories(self) -> None:
         covdata = DebugCoverageData()
         msg = "Couldn't combine from non-existent path 'xyzzy'"
         with pytest.raises(NoDataError, match=msg):
             combine_parallel_data(covdata, data_paths=['xyzzy'])
 
-    def test_interleaved_erasing_bug716(self):
+    def test_interleaved_erasing_bug716(self) -> None:
         # pytest-cov could produce this scenario. #716
         covdata1 = DebugCoverageData()
         covdata2 = DebugCoverageData()
@@ -888,7 +903,7 @@ class CoverageDataFilesTest(CoverageTest):
             ("[3-1]", "[b-a]"),
         ],
     )
-    def test_combining_with_crazy_filename(self, dpart, fpart):
+    def test_combining_with_crazy_filename(self, dpart: str, fpart: str) -> None:
         dirname = f"py{dpart}"
         basename = f"{dirname}/.coverage{fpart}"
         os.makedirs(dirname)
@@ -907,7 +922,7 @@ class CoverageDataFilesTest(CoverageTest):
         assert_measured_files(covdata3, MEASURED_FILES_1_2)
         self.assert_file_count(glob.escape(basename) + ".*", 0)
 
-    def test_meta_data(self):
+    def test_meta_data(self) -> None:
         # The metadata written to the data file shouldn't interfere with
         # hashing to remove duplicates, except for debug=process, which
         # writes debugging info as metadata.
@@ -934,7 +949,7 @@ class DumpsLoadsTest(CoverageTest):
     run_in_temp_dir = False
 
     @pytest.mark.parametrize("klass", [CoverageData, DebugCoverageData])
-    def test_serialization(self, klass):
+    def test_serialization(self, klass: TCoverageData) -> None:
         covdata1 = klass(no_disk=True)
         covdata1.add_lines(LINES_1)
         covdata1.add_lines(LINES_2)
@@ -945,7 +960,7 @@ class DumpsLoadsTest(CoverageTest):
         assert_line_counts(covdata2, SUMMARY_1_2)
         assert_measured_files(covdata2, MEASURED_FILES_1_2)
 
-    def test_misfed_serialization(self):
+    def test_misfed_serialization(self) -> None:
         covdata = CoverageData(no_disk=True)
         bad_data = b'Hello, world!\x07 ' + b'z' * 100
         msg = r"Unrecognized serialization: {} \(head of {} bytes\)".format(
@@ -961,7 +976,7 @@ class NoDiskTest(CoverageTest):
 
     run_in_temp_dir = False
 
-    def test_updating(self):
+    def test_updating(self) -> None:
         # https://github.com/nedbat/coveragepy/issues/1323
         a = CoverageData(no_disk=True)
         a.add_lines({'foo.py': [10, 20, 30]})
